/*-
 * Copyright (C) @BABOLO  2002 Dec 04
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef lint
static const char copyright[] = "\
@(#)Copyright (C) @BABOLO  2002 Dec 04\n\
@(#)All rights reserved.\n";
static const char rcsid[] = "$Id: shurecon.c,v 1.7 2006/11/09 14:29:15 babolo Exp $";
#endif /* not lint */

#include <sys/types.h>
#include <stdlib.h>
#include <stdio.h>
#include <babolo/BLINflag.h>
#include <mife.h>
#include <err.h>
#include <libpq-fe.h>
#include "pgoblin.h"

#define PRINTNOTNULL(A) !(A) ? "" : (A)

static void
NoticeProcessor(void *options, const char *message) {
    ifBLIN_QV1(((pgoblin_con *)options)->flag) fprintf(stderr, "%s", message);
}

int
pgoblin_shurecon(pgoblin_con *options) {
    PQconninfoOption *popt;
    int i;
    if  (options->conn != NULL && PQstatus(options->conn) == CONNECTION_OK) return(0);
    ifBLIN_QV4(options->flag) {
        fprintf( stderr
               , "Connection defaults:\n"
                 "size=chr keyword	envvar		compiled val		label\n"
               );
        popt = PQconndefaults();
        for (i = 0; popt[i].keyword; i++) {
            fprintf( stderr
                   , "%4d=%s~	%s%s%s%s%s%s%s%s%s\n"
                   , popt[i].dispsize
                   , popt[i].dispchar
                   , popt[i].keyword
                   , strlen(popt[i].keyword) < 8 ? "		" : "	"
                   , popt[i].envvar
                   , strlen(popt[i].envvar) < 8
                   ? "		"
                   : strlen(popt[i].envvar) < 16 ? "	" : " "
                   , PRINTNOTNULL(popt[i].compiled)
                   , !(popt[i].compiled) ? "	" : strlen(popt[i].compiled) < 7 ? "~	" : "~ "
                   , PRINTNOTNULL(popt[i].val)
                   , !(popt[i].val)
                   ? "		"
                   : strlen(popt[i].val) < 7 ? "~		" : "~	"
                   , popt[i].label
                   );
        }
        PQconninfoFree(popt);
            fprintf( stderr
                   , "PGREALM=%s%s PGDATESTYLE=%s%s PGTZ=%s%s PGCLIENTENCODING=%s%s PGGEQO=%s%s\n"
                   , PRINTNOTNULL(getenv("PGREALM"))
                   , getenv("PGREALM") ? "~" : ""
                   , PRINTNOTNULL(getenv("PGDATESTYLE"))
                   , getenv("PGDATESTYLE") ? "~" : ""
                   , PRINTNOTNULL(getenv("PGTZ"))
                   , getenv("PGTZ") ? "~" : ""
                   , PRINTNOTNULL(getenv("PGCLIENTENCODING"))
                   , getenv("PGCLIENTENCODING") ? "~" : ""
                   , PRINTNOTNULL(getenv("PGGEQO"))
                   , getenv("PGGEQO") ? "~" : ""
                   );
    }
    options->conn = PQsetdbLogin( options->host
                                , options->port
                                , NULL, NULL
                                , options->dbname
                                , options->username
                                , NULL
                                );
    ifBLIN_QV4(options->flag) {
        fprintf(stderr, "PQoptions=%s~\n", PQoptions(options->conn));
        fprintf( stderr, "PQdb=%s~ PQuser=%s~ PQpass=%s%s PQhost=%s%s PQport=%s%s PQtty=%s%s\n"
               , PQdb(options->conn)
               , PQuser(options->conn)
               , PRINTNOTNULL(PQpass(options->conn))
               , PQpass(options->conn) ? "~" : ""
               , PRINTNOTNULL(PQhost(options->conn))
               , PQhost(options->conn) ? "~" : ""
               , PRINTNOTNULL(PQport(options->conn))
               , PQport(options->conn) ? "~" : ""
               , PRINTNOTNULL(PQtty(options->conn))
               , PQtty(options->conn) ? "~" : ""
               );
        fprintf( stderr
               , "server_version=%s%s"
                 "client_encoding=%s%s"
                 "is_superuser=%s%s"
                 "session_authorization=%s%s"
                 "DateStyle=%s%s\n"
               , PRINTNOTNULL(PQparameterStatus(options->conn, "server_version"))
               , PQparameterStatus(options->conn, "server_version") ? "~ " : " "
               , PRINTNOTNULL(PQparameterStatus(options->conn, "client_encoding"))
               , PQparameterStatus(options->conn, "client_encoding") ? "~ " : " "
               , PRINTNOTNULL(PQparameterStatus(options->conn, "is_superuser"))
               , PQparameterStatus(options->conn, "is_superuser") ? "~ " : " "
               , PRINTNOTNULL(PQparameterStatus(options->conn, "session_authorization"))
               , PQparameterStatus(options->conn, "session_authorization") ? "~ " : " "
               , PRINTNOTNULL(PQparameterStatus(options->conn, "DateStyle"))
               , PQparameterStatus(options->conn, "DateStyle") ? "~" : ""
               );
        fprintf( stderr
               , "PQprotocolVersion=%d PQsocket=%d PQbackendPID=%d\n"
               , PQprotocolVersion(options->conn)
               , PQsocket(options->conn)
               , PQbackendPID(options->conn)
               );
    }
    ifBLIN_QV3(options->flag) fprintf(stderr, "PQstatus %d\n", PQstatus(options->conn));
    PQsetNoticeProcessor(options->conn, NoticeProcessor, options);
    if  (PQstatus(options->conn) != CONNECTION_BAD) return(0);
    ifBLIN_QV1(options->flag)
        warnx("Connection to PostgreSQL failed: %s\n", PQerrorMessage(options->conn));
    return(1);
}
